#!/bin/bash


## Messenger filenames
FILE_DATE=mesg_01_date_$PATTERNAME.txt    ; FILE_STAT=mesg_02_status_$PATTERNAME.txt
FILE_RESU=mesg_03_result_$PATTERNAME.txt  ; FILE_NEMO=mesg_04_nemogcm_$PATTERNAME.txt
FILE_XIOS=mesg_05_xios_$PATTERNAME.txt    ; FILE_COMP=mesg_06_compiler_$PATTERNAME.txt
FILE_MPIN=mesg_07_mpi_$PATTERNAME.txt     ; FILE_NCDF=mesg_08_netcdf_$PATTERNAME.txt
FILE_INPT=mesg_09_inputs_$PATTERNAME.txt  ; FILE_CPUT=mesg_10_cputime_$PATTERNAME.txt
FILE_COMM=mesg_11_comments_$PATTERNAME.txt;

## 
FILE_TRUS=trusting_${DATE}_$PATTERNAME.txt; FILE_ARCH=trusting_${DATE}_$PATTERNAME.tar.gz

print_step() { printf "\nStep.....\n$1\n"; }

init_files() {
    echo 'Date'        > ${FILE_DATE}; echo 'Status'           > ${FILE_STAT}
    echo 'Result'      > ${FILE_RESU}; echo 'NEMOGCM rev.'     > ${FILE_NEMO}
    echo 'XIOS rev.'   > ${FILE_XIOS}; echo 'Fortran compiler' > ${FILE_COMP}
    echo 'MPI libs'    > ${FILE_MPIN}; echo 'NetCDF libs'      > ${FILE_NCDF}
    echo 'Input files' > ${FILE_INPT}; echo 'Real CPU time'    > ${FILE_CPUT}
    echo 'Comments'    > ${FILE_COMM}
}

get_date() {
    local dat=$( date -ud ${DATE} +"%F %R %Z" )

    echo $dat >> ${FILE_DATE}
}

get_nemo_rev() {
    local rev_loc svn_cmd
    local rev=0 svn_opt='status'

    ## If -v|--version option has been set, modify svn command
    if   [ $( echo ${NEMO_VERS} | grep  "HEAD\|up\|update"                     ) ]; then
	svn_cmd='svn update -r HEAD'
    elif [ $( echo ${NEMO_VERS} | grep -o '{[0-9]\{4\}-[0-9]\{2\}-[0-9]\{2\}}' ) ]; then
	svn_cmd='svn update -r '$( echo ${NEMO_VERS} | grep -o '{[0-9]\{4\}-[0-9]\{2\}-[0-9]\{2\}}' )
    elif [ $( echo ${NEMO_VERS} | grep -o '[0-9]*'                             ) ]; then
	svn_cmd='svn update -r '$( echo ${NEMO_VERS} | grep -o '[0-9]*' )
    else
	svn_cmd='svn '${svn_opt}
    fi

    for dir in ${NEMO_ARCH} ${NEMO_CONF} ${NEMO_ENGI} \
               ${NEMO_EIOI} ${NEMO_EFCM}              \
	       ${NEMO_TCMP} ${NEMO_TRBD}              \
	       ${DIR_XIOS}                             ; do

	## For time being, just get revision number from XIOS
	if [ "$dir" == "${DIR_XIOS}" ]; then
	    rev_loc=$( svn info $dir | awk '(NR == 9) {print $NF}' )
	    echo 'XIOS '${rev_loc} >> model.log
	    echo "<a href=\"https://forge.ipsl.jussieu.fr/ioserver/changeset/${rev_loc}\" target=\"_blank\">${rev_loc}</a>" \
		>> ${FILE_XIOS}
	    continue
	fi

	echo $dir
	${svn_cmd} $dir
	rev_loc=$( svn info $dir | awk '(NR == 9) {print $NF}' )
	(( ${rev_loc} >= $rev )) && rev=${rev_loc}
    done

    echo 'NEMOGCM '$rev   >> model.log
    echo "<a href=\"https://forge.ipsl.jussieu.fr/nemo/changeset/$rev\" target=\"_blank\">$rev</a>" \
	>> ${FILE_NEMO}
}

get_soft_rel() {
    local soft_rel

    for soft in $CDO $COMPILER $MPI $NETCDF; do
	soft_rel=''
        ((   ${I_MODULE} == 1   )) && soft_rel=$( echo $LOADEDMODULES | sed "s/.*$soft\/\([0-9.a-z_]*\).*/\1/i" )
	[    -z "${soft_rel}"    ] && soft_rel=$( echo $PATH          | sed "s/.*$soft\([0-9.a-z_]*\).*/\1/i"   )
	[ "$soft" == "$COMPILER" ] && soft_rel=$( $soft --version     | grep -m1 -oe '\<[0-9. ]*\>'             )
	## Cleaning characters string to display proper soft name
	soft=$( echo $soft | sed 's#\\##g; s#[/-]$##' )
	echo $soft ${soft_rel} >> model.log
    done

    sed -n 4p model.log > ${FILE_COMP}; sed -n 5p model.log > ${FILE_MPIN}; sed -n 6p model.log > ${FILE_NCDF}
}

get_inputs() {
    local cmd_iol="tar -tvf ${NEMO_FORC}/${NEMO_TARF}"; cmd_iof="tar -vxf ${NEMO_FORC}/${NEMO_TARF}"

    [ -z "${NEMO_TARF}" ] && cmd_iol="ls ${NEMO_FORC}/*" && cmd_iof="\cp ${NEMO_FORC}/* ."
    ${cmd_iol} > inputs_list.txt && ${cmd_iof} > /dev/null
}

diff_inputs() {
    local dif
    local mesg='Same'

    for file in inputs_list.txt $( ls namelist_* ) $( ls *.xml ); do
	dif=''
	if [ -e ${REFE_DIR}/$file ]; then dif=$( diff -q $file ${REFE_DIR}/$file ); else dif=0; fi
	if [ -n "$dif" ]; then mesg='Different'; [ "$dif" != '0' ] && echo $dif | tee -a ${FILE_COMM}; fi
    done

    echo $mesg | tee -a ${FILE_INPT}
}

job_pending() {
    local time_elapsed=0; time_increment=30

    sleep ${time_increment}

    while [[ $( ${JOB_LIST} | grep ${JOB_ID} ) && ${time_elapsed} -lt ${TIME_LIMI} ]]; do
	printf "\n####################################################\n" >> computation.log
	${JOB_INFO} ${JOB_ID}                                             >> computation.log
	sleep ${time_increment}
	time_elapsed=$(( ${time_elapsed} + ${time_increment} ))
    done

    sleep ${time_increment}

    if [ ${time_elapsed} -eq ${TIME_LIMI} ]; then
	${JOB_DELE} ${JOB_ID} &> /dev/null
	TIME_LIMIT=$(( ${TIME_LIMI} / 3600 ))
#	export TIME_LIMIT
	get_out 6
    fi
}

diff_results() {
    for file in ocean.output $( ls *.stat ); do
	[ ! -e ${REFE_DIR}/$file ] && export ST='FAILED' && get_out 7
	diff -q $file ${REFE_DIR}/$file | tee -a ${FILE_COMM}
	(( $? != 0 )) && export ST='FAILED'
    done
}

diff_restart() {
    local base_name file nb_dom diff_rcd

    [ ! -e  ${REFE_DIR}/time.step ] && export ST='FAILED' && get_out 9

#    export LAST_TIME_STEP=$( cat ${REFE_DIR}/time.step | tr -d [:space:] )
    LAST_TIME_STEP=$( cat ${REFE_DIR}/time.step | tr -d [:space:] )
    echo 'Last time step of standard run: '${LAST_TIME_STEP}

    if [ $( find -regex ".*_0+${LAST_TIME_STEP}_restart.*\.nc" -print -quit ) ]; then
	base_name=$( find -regex ".*_0+${LAST_TIME_STEP}_restart.*\.nc" -print -quit | awk -F/ '{print $NF}' \
	             | sed "s/^\(.*\)$LAST_TIME_STEP\_restart.*/\1$LAST_TIME_STEP\_/"                         )

	for comp in restart restart_ice restart_trc; do
	    file=${base_name}${comp} && printf "$file.nc: "

	    nb_dom=$( find -name "${file}_[0-9]*.nc" | wc -l | awk '{print $1}' )
	    if   [ ${nb_dom} -gt 1 ]; then
		${NEMO_TRBD}/rebuild_nemo -t $NPROC $file ${nb_dom} > /dev/null
		[ $? -eq 0 ] && rm -f ${base_name}${comp}_[0-9]*.nc > /dev/null
	    elif [ ${nb_dom} -eq 0 ]; then
		export ST='FAILED' && get_out 10
	    fi

	    if [ -e ${REFE_DIR}/$file.nc ]; then
	    ## UNIX `cmp` not suitable (filename & timestamp in .nc file)
#		diff_rcd=$( $CDO diffn $file.nc ${REFE_DIR}/$file.nc 2> /dev/null \
#		           | awk '/records/ {print $1}'                           )
		diff_rcd=$( $CDO diffn $file.nc ${REFE_DIR}/$file.nc 2> /dev/null \
		           | awk '/records/ {print $0}'                           )

#		if [[ ! -z ${diff_rcd} && ${diff_rcd} -ne 0 ]]; then
		if [[ ! -z ${diff_rcd} && $( echo ${diff_rcd} | awk '{print $1}') -ne 0 ]]; then
		    export ST='FAILED'
		    echo $CDO $( echo ${diff_rcd} | sed 's/differ$/differ,/' | tr -d '\n' )
#		    printf "$CDO ${diff_rcd}\n"
		else
		    echo 'identical'
#		    printf "identical\n"
		fi

	    else
		export ST='FAILED' && get_out 9
	    fi

	done

    else
	export ST='FAILED'
    fi

}

get_cpu_time() {
    local real_cpu_time=$( eval ${JOB_TIME} )

    echo ${real_cpu_time} | tee -a ${FILE_CPUT}
}

comments() {
    local state=$1

    if [ "$state" == 'E R R O R' ]; then
	[ -e time.step ] && LAST_TIME_STEP=$( cat time.step | tr -d ' ' )
#	export LAST_TIME_STEP
	[ -e ocean.output ] && line=$( grep -m1 -A4 "$state"   ocean.output | tr -d '\n' )
	echo $line >> ${FILE_COMM}
    else
	[ -e ocean.output ] && line=$( grep -m1 -A2 "^ $state" ocean.output | tr -d '\n' )
    fi

    echo $line
}

mesg_make() {
    paste -d ';' mesg_*.txt > ${FILE_TRUS}
    cat                       ${FILE_TRUS}
}

mesg_publish() {
    local cmd

    if [ $PUBLISH -eq 1 ]; then

	if [ -f ${REFE_DIR}/trusting_$PATTERNAME.txt ]; then cmd='tail -1'; else cmd='cat'; fi

	$cmd ${FILE_TRUS} >> ${REFE_DIR}/trusting_$PATTERNAME.txt

  ## Send mail only when FAILED
	if [[ ! -z $EMAIL && "$ST" == 'FAILED' ]]; then
	    cat << END_MAIL > trusting.mail
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

Dear $USER,

The trusting sequence for ${REFE_CONF} has failed.
Directory: ${TEST_DIR}
Archive created: ${FILE_ARCH} in ${REFE_DIR}

XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

END_MAIL
	    [ -e ${TEST_DIR}/${FILE_TRUS} ] && cat ${TEST_DIR}/${FILE_TRUS} >> trusting.mail
	    mail -s "[trusting ${REFE_CONF} $BRANCH] $ST $ERR" $EMAIL       <  trusting.mail
	fi

    fi
}

get_out() {
    ERR=$1

    cd ${TEST_DIR}
    echo $ST >> ${FILE_STAT}

    if [ "$ST" == 'FAILED' ]; then

        ## Error identification
	case $ERR in
	    ## Compilation
	    1) ERR='XIOS compilation failed'               ;;  2) ERR='NEMO compilation failed'            ;;
	    ## Submission
	    3) ERR='Missing input files'                   ;;  4) ERR='Job submission error'               ;;
	    ## Running
	    5) ERR='Crashed at time step '${LAST_TIME_STEP};;  6) ERR='Exceeded time limit '${TIME_LIMI}'h';;
	    ## Results
	    7) ERR='Missing previous outputs '             ;;  8) ERR='New outputs  differ/missing'        ;;
	    9) ERR='Missing previous restarts'             ;; 10) ERR='New restarts differ/missing'        ;;
	    ## Undefined
	    *) ERR='Unknown error'                         ;;
	esac

	export ERR
    fi

    echo $ERR >> ${FILE_RESU}

    ## Save tested configuration if trusting failed
    if [[ "$ST" == 'FAILED' && $DEBUG -eq 0 ]]; then
	echo 'Creating archive '${FILE_ARCH}' under '${REFE_DIR}
	tar -czf ${REFE_DIR}/${FILE_ARCH} ../*
    fi

    mesg_make
    mesg_publish

    exit 0
}
